#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

int main(int argc, char* argv[]) 
{  
	int n;
	double h, x, sum;
	double dt;
	int myid, size;
	double mysum, mystart, myend;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &myid);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	if (argc > 1)
		n = atoi(argv[1]);
	else
		n = 1;
	
	h = 1.0/n/1000000;

	MPI_Barrier(MPI_COMM_WORLD);
	dt = MPI_Wtime();

	mysum = 0.0;
	mystart = myid/(double)size;
	myend = (myid+1)/(double)size;
	x = mystart;
	while (x < myend)
	{
		mysum += 4.0/(1+x*x)*h;
		x += h;
	}

	MPI_Reduce(&mysum, &sum, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

	dt = MPI_Wtime()-dt;
	if (myid == 0)
		printf("n = %d, Pi = %lf (%lf s)\n", n, sum, dt);

	MPI_Finalize();

	return 0;
}
